#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>
#include <signal.h>
#include <X11/keysym.h>
#include <string.h>
#include "wtb.h"
#include "text.h"
#include "x_util.h"
#include "kanji.h"
#include "KdWindow.h"
#include "dhm2.h"
#include "MainWindow.h"

struct CARDTYPE CardType[] = {
	{ CT_FIGHTER,
	  1,  6, 2, +7,
	  "",
	  "ϺǹΡǤäϤʤʡ"},
	{CT_FIGHTER,
	 1, 10, 3, -3,
	 "Ĺ⡡",
	 "ǯСäĤʡʾС"},
	{CT_FIGHTER,
	 1,  9, 2, +3,
	 "˻",
	 "ϡϤȤХ󥹤ΤȤ줿ɤ̼Ǥ"},
	{CT_FIGHTER,
	 1, 10, 1, +5,
	 "ܡ",
	 "㤬Τ˴򤫤Ƥʤ"},
	{CT_FIGHTER,
	 1, 11, 3, -1,
	 "ܶΤ",
	 "Ĥϰ̼Ǥɤ̼Ǥ"},
	{CT_FIGHTER,
	 1,  8, 2, +4,
	 "ΦӸ",
	 "ȯΰϾ֥ޤ̼ʾСˡ"},
	{CT_FIGHTER,
	 1, 12, 2, +6,
	 "",
	 "ǶΡˣ繥"},

	{CT_MAGE   ,
	 1,  7, 3,  0,
	 "ѥƥ",
	 "ͤʤΤǤ̣ʥ֤夿ʤ"},
	{CT_MAGE   ,
	 1,  6, 3, +5,
	 "ͥ",
	 "Ϻǹɸ椫Ϥ㤯㡣"},
	{CT_MAGE   ,
	 1,  8, 1, +8,
	 "󥷥",
	 "¤ʹǳμ¤ˡܤ̼ʤΤǤ"},
	{CT_MAGE   ,
	 1,  9, 3, -2,
	 "ꥷ",
	 "ˡعǤӤξ塩"},
	{CT_MAGE   ,
	 1, 10, 2, +3,
	 "ե",
	 "Υȥ졼˥󥰤ΤޤΡϤϤ롣"},
	{CT_MAGE   ,
	 1,  7, 3, +1,
	 "Υ",
	 "ɼˡ繥ʴʽλҡ"},
	{CT_MAGE   ,
	 1, 11, 3, +2,
	 "ƥ",
	 "ܤ餹ȰֶۡǥʡΤꡣ"},

	{CT_MONSTER,
	 1, 10, 1, +3,
	 "ؤؤ",
	 "ʪǤѤʹ򤷤ޤؤؤ"},
	{CT_MONSTER,
	 1, 15, 1,  0,
	 "ؤؤ",
	 "ʪǤѤʹ򤷤ޤ"},
	{CT_MONSTER,
	 1, 12, 1, +1,
	 "ؤؤ",
	 "ʪǤѤʹ򤷤ޤ"},
	{CT_MONSTER,
	 1, 12, 2, -1,
	 "إ顼",
	 "¤ʪʾС"},
	{CT_MONSTER,
	 1, 14, 2,  0,
	 "˿˼",
	 "ⲿ褫ǯࡣ"},
	{CT_MONSTER,
	 1, 15, 2, +2,
	 "ȥ",
	 "餬ȥ͡С"},

	{CT_ATTACK,
	 4, -1, 0,  0,
	 "ä",
	 "äפǤäˤǤߤʻȤޤ"},
	{CT_ATTACK,
	 4,  0, 0,  0,
	 "ѡ",
	 "ѤǤʤʤˤǤߤʻȤޤ"},
	{CT_ATTACK,
	 4, +1, 0,  0,
	 "ä",
	 "äǤˤǤߤʻȤޤ"},

	{CT_ATTACK|CT_WEAPON,
	 3, -1, 0,  0,
	 "ɤäϤޤ",
	 "ȾˤǤΤȤޤ"},
	{CT_ATTACK|CT_WEAPON,
	 3,  0, 0,  0,
	 "",
	 "ȤäˤǤΤȤޤ"},
	{CT_ATTACK|CT_WEAPON,
	 3, +1, 0,  0,
	 "ؤη",
	 "ȤˤǤΤȤޤ"},

	{CT_ATTACK|CT_MAGIC,
	 3, -1, 0,  0,
	 "",
	 "ˤˡǤˡȤȤޤ"},
	{CT_ATTACK|CT_MAGIC,
	 3,  0, 0,  0,
	 "",
	 "ˤˡǤˡȤȤޤ"},
	{CT_ATTACK|CT_MAGIC,
	 3, +1, 0,  0,
	 "㤤",
	 "ˤˡǤˡȤȤޤ"},

	{CT_ATTACK|CT_WEAPON|CT_DOUBLE,
	 1, -1, 0,  0,
	 "֤Ϥޤ",
	 "ȾˤǤ⣲ˤǤΤȤޤ"},
	{CT_ATTACK|CT_WEAPON|CT_DOUBLE,
	 1,  0, 0,  0,
	 "ޤ",
	 "ȤäˤǤ⣲ˤǤΤȤޤ"},
	{CT_ATTACK|CT_WEAPON|CT_DOUBLE,
	 1, +1, 0,  0,
	 "ؤؤη",
	 "ȤˤǤ⣲ˤǤΤȤޤ"},

	{CT_ATTACK|CT_MAGIC|CT_DOUBLE,
	 1, -1, 0,  0,
	 "Ƥ",
	 "ˤˡǤˡȤȤޤ"},
	{CT_ATTACK|CT_MAGIC|CT_DOUBLE,
	 1,  0, 0,  0,
	 "Ƥ",
	 "󤹤ˤˡǤˡȤȤޤ"},
	{CT_ATTACK|CT_MAGIC|CT_DOUBLE,
	 1, +1, 0,  0,
	 "㤤Ƥ",
	 "ˤˡǤˡȤȤޤ"},

	{CT_ATTACK|CT_MON|CT_EXT,
	 2, +2, 0, 0,
	 "ؤ󤬤",
	 "ʪ䤱ˤʤˤǤʪĤޤ"},
	{CT_ATTACK|CT_MON|CT_EXT,
	 2, +3, 0, 0,
	 "ؤ󤢤ä",
	 "ʪ䤱ˤʤФʤˤǤʪĤޤ"},
	{CT_ATTACK|CT_MON|CT_EXT,
	 2, +4, 0, 0,
	 "ؤܤФ",
	 "ʪ䤱ˤʤФˤǤʪĤޤ"},

	{CT_ATTACK|CT_WEAPON|CT_EXT,
	 1, +5, 0, 0,
	 "ưʸ",
	 "蘆ΤǤΤκǶǤ"},

	{CT_ATTACK|CT_MAGIC|CT_EXT,
	 1, 2, 0, 0,
	 "㤹",
	 "㤹ˤˡǤѻդκǶǤ"},

	{CT_SHIELD,
	 3, 3, 0, 0,
	 "ä㤤",
	 "Ǥä㤤Ǥ"},
	{CT_SHIELD,
	 3, 4, 0, 0,
	 "ä",
	 "ǤäǤ"},
	{CT_SHIELD,
	 3, 5, 0, 0,
	 "ˣ", "Ǥޤ"},

	{CT_SUPPORT,
	 2, 0, 0, 0,
	 "Ф",
	 "ŨΥɤ򤢤Фޤ"},
	{CT_SUPPORT,
	 10, 0, 0, 0,
	 "",
	 "ˤäȤɤޤ"},
	{CT_SUPPORT,
	 1, 0, 0, 0,
	 "ä㤳",
	 "Υ˥åȤ򤤤ޤ"},
	{CT_SUPPORT,
	 2, 0, 0, 0,
	 "ɤ",
	 "Ũι̵ˤޤ"},
	{CT_SUPPORT,
	 1, 0, 0, 0,
	 "Τ",
	 "˥åȤλ˴̵ˤޤ"},

	{CT_OPEN,
	 1, 0, 0, 0,
	 "Ũƨ˴",
	 "ˤäȤɤʤʤޤ̤ʤƻǤ"},
	{CT_OPEN,
	 2, 0, 0, 0,
	 "դĤ줿",
	 "٤"},
	{CT_OPEN,
	 2, 0, 0, 0,
	 "á",
	 "󥫡ɤʤʤޤ"},

	{0, 1, 0, 0, 0, ""}		/* DUMMY */
};


int Card[MAX_CARD];
int CharStack[MAX_CHAR];
int Hand[2][5];
int CdOpen[2][5];
int Char[2][5];
int CardPt = 0;
int CardDropPt = 0;
int CharPt = 0;
int CharDropPt = 0;
int HandNum[2];
int CardNum;
int CharNum;
int Chars[2];
char pflag[MAX_ITEM];

Display* dis;  // Default Display
int scr;  // Default Screen
Window rootwin;
int screen_depth;  // screen depth
GC gc;  // utility gc
clock_t interval;

Pixmap stage_pm;
Pixmap dice_pm[6];  // 
Pixmap caut_pm; // ٹ
Pixmap info_pm; // 

void make_path(char* path, char* fname)
{
	int i;
	
	strcpy(path, DATA_DIR);
	if (path[strlen(path)-1]!='/') {
		strcat(path, "/");
	}
	strcat(path, fname);
	for(i=0; path[i]; i++) {
		if (iskanji(path[i])) {
			i++;
		} else {
			path[i] = tolower(path[i]);
		}
	}
}



int main(void)
{
    Window win;
    XSetWindowAttributes att;
    XEvent ev;
	int ret;
	int width, height;
	clock_t last_clk;
	int f, i;
	char path[256];
	char buf[256];
	XGCValues gv;
	MainWindow* main_win;
	
	dis = XOpenDisplay(NULL);  // displayΥץ
	if(dis==NULL) {
		fprintf(stderr, "Can't Open Display\n");
		exit(1);
	}
	scr = DefaultScreen(dis);
	rootwin = DefaultRootWindow(dis);
	screen_depth = DefaultDepth(dis, scr);
	printf("screen_depth = %d\n", screen_depth);

    win = XCreateSimpleWindow(// ɥΥץ
		dis,
		rootwin,
		100, 100,
		640, 400,
		5,
		BlackPixel(dis,scr),
		WhitePixel(dis,scr));
	KdWindow::win = win;
		
    att.backing_store = WhenMapped;
    XChangeWindowAttributes(dis, win, CWBackingStore, &att);

	gv.graphics_exposures = False;
	gc = XCreateGC(dis, win, GCGraphicsExposures, &gv);
	XStoreName(dis, win, "äHʥɥࡢʤ󤹤?");
    XSelectInput(dis, win, ExposureMask | ButtonPressMask | KeyPressMask);
    XMapWindow(dis, win);
	XFlush(dis);

	do {
		XNextEvent(dis, &ev);
	} while(ev.type!=Expose);

	setup_font();

	make_path(path, "waku.wtb");
	loadWTB(&stage_pm, win, path, &width, &height);
	for(i=1; i<=6; i++) {
		sprintf(buf, "dice%d.wtb", i);
		make_path(path, buf);
		loadWTB(&dice_pm[i-1], win, path, &width, &height);
	}
	make_path(path, "caution.wtb");
	loadWTB(&caut_pm, win, path, &width, &height);
	make_path(path, "info.wtb");
	loadWTB(&info_pm, win, path, &width, &height);
	
	main_win = new MainWindow(NULL);
	KdWindow::add_window(main_win);

	XSelectInput(dis, win, ButtonPressMask | KeyPressMask);
	last_clk = clock();

	while(1) {
		f = KD_EV_NONE;
		if (XCheckMaskEvent(dis,
							ExposureMask | KeyPressMask | ButtonPressMask,
							&ev)) {
			if(ev.type==KeyPress) {
				f = KD_EV_KEY;
			}
			else if(ev.type==ButtonPress) {
				f = KD_EV_MOUSE;
			}
			else if(ev.type==Expose) {
				f = KD_EV_EXPOSE;
			}
		}
		else if(clock() - last_clk >= interval) {
			last_clk = clock();
			f = KD_EV_TIMER;
		}
		if (f!=KD_EV_NONE) {
			ret = KdWindow::exec_event(f, &ev);
			if(ret==ST_BREAKGAME) break;
		}
	}

	setdown_font();

	XFreeGC(dis, gc);
	XDestroyWindow(dis, win);
	XCloseDisplay(dis);

    return(0);
}


